==> ThbgmExtractor-1.6.0.tar.bz2: ThbgmExtractor-1.6.0/core/Smdn.Windows.Forms.ThbgmPlayer/Smdn.Windows.Forms.ThbgmPlayer/Player.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2009-2013 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Drawing;
using System.IO;
using System.Threading;
using System.Windows.Forms;

using Smdn.Formats.Thbgm;
using Smdn.Media;

namespace Smdn.Windows.Forms.ThbgmPlayer {
  public partial class Player : UserControl {
    /*
     * class members
     */
    public const int MaxVolume = 100;
    public const int MinVolume = 0;

    static Player()
    {
      imageButtonPlay = new Bitmap(24, 24);
      imageButtonStop = new Bitmap(24, 24);
      imageButtonPause = new Bitmap(24, 24);

      var images = new[] {
        imageButtonPlay, imageButtonStop, imageButtonPause,
      };

      for (var i = 0; i < images.Length; i++) {
        var image = images[i];

        using (var g = Graphics.FromImage(image)) {
          g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
          g.Clear(Color.Transparent);

          switch (i) {
            case 0:
              var trianglePoints = new[] {
                new PointF(image.Width * 0.2f, image.Height * 0.2f),
                new PointF(image.Width * 0.2f, image.Height * 0.8f),
                new PointF(image.Width * 0.8f, image.Height * 0.5f),
              };
              g.FillPolygon(SystemBrushes.ControlText, trianglePoints);
              break;

            case 1:
              var rectangle = RectangleF.FromLTRB(image.Width * 0.2f, image.Height * 0.2f, image.Width * 0.8f, image.Height * 0.8f);
              g.FillRectangle(SystemBrushes.ControlText, rectangle);
              break;

            case 2:
              var leftRectangle  = RectangleF.FromLTRB(image.Width * 0.2f, image.Height * 0.2f, image.Width * 0.4f, image.Height * 0.8f);
              var rightRectangle = RectangleF.FromLTRB(image.Width * 0.6f, image.Height * 0.2f, image.Width * 0.8f, image.Height * 0.8f);
              g.FillRectangle(SystemBrushes.ControlText, leftRectangle);
              g.FillRectangle(SystemBrushes.ControlText, rightRectangle);
              break;
          }
        }
      }
    }

    private static Bitmap imageButtonPlay;
    private static Bitmap imageButtonStop;
    private static Bitmap imageButtonPause;

    /*
     * events
     */
    public event EventHandler PlayDone;
    public event EventHandler TrackChanged;

    /*
     * properties
     */
    public TimeSpan Position {
      get
      {
        RejectDisposed();
        if (player == null)
          return TimeSpan.MinValue;
        else
          return player.Position;
      }
      set
      {
        RejectDisposed();
        if (player != null)
          player.Position = value;
      }
    }

    public Track Track {
      get
      {
        RejectDisposed();
        return track;
      }
      set
      {
        RejectDisposed();
        OnTrackChanged(value);
      }
    }

    public string BgmSourcePath {
      get { return bgmSourcePath; }
      set
      {
        if (value == bgmSourcePath)
          return;

        bgmSourcePath = value;

        Stop();

        buttonPlay.Enabled = File.Exists(bgmSourcePath);

        waveFormView.BgmSourcePath = BgmSourcePath;
      }
    }

    public int Volume {
      get { return trackBarVolume.Value; }
      set { trackBarVolume.Value = value; }
    }

    public bool Repeat {
      get { return checkBoxRepeat.Checked; }
      set
      {
        RejectDisposed();

        checkBoxRepeat.Checked = value;

        if (player != null)
          player.Repeat = value;
      }
    }

    /*
     * construction/destruction
     */
    public Player()
    {
      InitializeComponent();

      player = PcmStreamPlayer.Create();
      player.PlayDone += delegate {
        DisableStopButton();
        OnPlayDone(EventArgs.Empty);
      };

      waveFormView.SeekBarMoved += delegate(object sender, SeekBarMovedEventArgs e) {
        if (player == null || !player.Playing)
          e.Cancel = true;
        else
          player.Position = e.SeekBarPosition;
      };

      trackBarVolume.Minimum = MinVolume;
      trackBarVolume.Maximum = MaxVolume;
      trackBarVolume.SmallChange = (MaxVolume - MinVolume) / 100;
      trackBarVolume.LargeChange = (MaxVolume - MinVolume) / 10;
      trackBarVolume.Value = (MaxVolume - MinVolume) * 3 / 4;
      trackBarVolume.ValueChanged += delegate {
        SetPlayerVolume();
      };

      buttonPlay.Click += delegate {
        if (player == null || !player.Playing)
          Play();
        else if (player.Pausing)
          Resume();
        else
          Pause();
      };

      buttonPlay.Image = imageButtonPlay;
      buttonPlay.ImageAlign = ContentAlignment.MiddleCenter;

      buttonStop.Click += delegate { Stop(); };
      buttonStop.Image = imageButtonStop;
      buttonStop.ImageAlign = ContentAlignment.MiddleCenter;
      buttonStop.Enabled = false;

      checkBoxRepeat.CheckedChanged += delegate {
        if (player != null)
          player.Repeat = checkBoxRepeat.Checked;
      };

      SetPlayerVolume();

      timer = new System.Timers.Timer(100);
      timer.Enabled = false;
      timer.AutoReset = true;
      timer.SynchronizingObject = labelPosition;
      timer.Elapsed += delegate { SetSeekBarPosition(); };
    }

    protected override void OnLoad(EventArgs e)
    {
      timer.Start();

      base.OnLoad(e);
    }

    private void DisposeInternal(bool disposing)
    {
      if (disposing) {
        if (!disposed) {
          Stop();

          if (timer != null) {
            timer.Stop();
            timer.Dispose();
            timer = null;
          }

          if (player != null) {
            player.Close();
            player = null;
          }

          if (stream != null) {
            stream.Close();
            stream = null;
          }

          disposed = true;
        }
      }
    }

    /*
     * public methods
     */
    public void Play()
    {
      RejectDisposed();

      if (track == null)
        return;

      buttonPlay.Image = imageButtonPause;
      buttonStop.Enabled = true;

      try {
        stream = track.GetStream(BgmSourcePath);
      }
      catch (FileNotFoundException) {
        throw;
      }
      catch (ArgumentException) {
        buttonPlay.Image = imageButtonPlay;
        buttonStop.Enabled = false;
        return;
      }

      SetPlayerVolume();

      player.PlayAsync(stream, stream.Format.GetWaveFormatEx(), checkBoxRepeat.Checked);
    }

    public void Stop()
    {
      RejectDisposed();

      if (player == null)
        return;

      DisableStopButton();

      player.Stop();
    }

    public void Pause()
    {
      RejectDisposed();

      if (player == null)
        return;

      buttonPlay.Image = imageButtonPlay;

      player.Pause();
    }

    public void Resume()
    {
      RejectDisposed();

      if (player == null)
        return;

      buttonPlay.Image = imageButtonPause;

      player.Resume();
    }

    /*
     * private methods
     */
    private void SetPlayerVolume()
    {
      if (player == null)
        return;

      var scale = (trackBarVolume.Value - trackBarVolume.Minimum) / (double)(trackBarVolume.Maximum - trackBarVolume.Minimum);

      player.Volume = (int)(100 * scale);
    }

    private void DisableStopButton()
    {
      if (InvokeRequired) {
        Invoke((MethodInvoker)DisableStopButton);
        return;
      }

      buttonPlay.Image = imageButtonPlay;
      buttonStop.Enabled = false;
    }

    private void SetSeekBarPosition()
    {
      if (player == null)
        return;

      var position = player.Position;

      labelPosition.Text = string.Format("{0}:{1:D2}.{2:D3}", (int)position.TotalMinutes, position.Seconds, position.Milliseconds);

      waveFormView.SeekBarPosition = position;
    }

    private void OnTrackChanged(Track newTrack)
    {
      if (track == newTrack)
        return;

      Stop();

      if (stream != null) {
        stream.Close();
        stream = null;
      }

      track = newTrack;

      waveFormView.Track = track;

      effectEditor.Effect = (track == null) ? null : track.Effect;

      OnTrackChanged(EventArgs.Empty);
    }

    protected virtual void OnTrackChanged(EventArgs e)
    {
      var ev = this.TrackChanged;

      if (ev != null)
        ev(this, e);
    }

    protected virtual void OnPlayDone(EventArgs e)
    {
      var ev = this.PlayDone;

      if (ev != null)
        ev(this, e);
    }

    private void RejectDisposed()
    {
      if (disposed)
        throw new ObjectDisposedException(GetType().Name);
    }

    private bool disposed = false;
    private PcmStreamPlayer player;
    private Track track = null;
    private string bgmSourcePath = null;
    private EffectAppliedBgmStream stream = null;
    private System.Timers.Timer timer = null;
  }
}

==> ThbgmExtractor-1.6.2.tar.bz2: ThbgmExtractor-1.6.2/core/Smdn.Windows.Forms.ThbgmPlayer/Smdn.Windows.Forms.ThbgmPlayer/Player.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2009-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Drawing;
using System.IO;
using System.Threading;
using System.Windows.Forms;

using Smdn.Formats.Thbgm;
using Smdn.Media;

namespace Smdn.Windows.Forms.ThbgmPlayer {
  public partial class Player : UserControl {
    /*
     * class members
     */
    public const int MaxVolume = 100;
    public const int MinVolume = 0;
    public const int DefaultVolume = 75;

    static Player()
    {
      imageButtonPlay = new Bitmap(24, 24);
      imageButtonStop = new Bitmap(24, 24);
      imageButtonPause = new Bitmap(24, 24);
      imageButtonPrev = new Bitmap(24, 24);
      imageButtonNext = new Bitmap(24, 24);

      var images = new[] {
        imageButtonPlay,
        imageButtonStop,
        imageButtonPause,
        imageButtonPrev,
        imageButtonNext,
      };

      for (var i = 0; i < images.Length; i++) {
        var image = images[i];

        using (var g = Graphics.FromImage(image)) {
          g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
          g.Clear(Color.Transparent);

          switch (i) {
            case 0: { // play
              var trianglePoints = new[] {
                new PointF(image.Width * 0.2f, image.Height * 0.2f),
                new PointF(image.Width * 0.2f, image.Height * 0.8f),
                new PointF(image.Width * 0.8f, image.Height * 0.5f),
              };
              g.FillPolygon(SystemBrushes.ControlText, trianglePoints);
              break;
            }

            case 1: { // stop
              var rectangle = RectangleF.FromLTRB(image.Width * 0.2f, image.Height * 0.2f, image.Width * 0.8f, image.Height * 0.8f);
              g.FillRectangle(SystemBrushes.ControlText, rectangle);
              break;
            }

            case 2: { // pause
              var leftRectangle  = RectangleF.FromLTRB(image.Width * 0.2f, image.Height * 0.2f, image.Width * 0.4f, image.Height * 0.8f);
              var rightRectangle = RectangleF.FromLTRB(image.Width * 0.6f, image.Height * 0.2f, image.Width * 0.8f, image.Height * 0.8f);
              g.FillRectangle(SystemBrushes.ControlText, leftRectangle);
              g.FillRectangle(SystemBrushes.ControlText, rightRectangle);
              break;
            }

            case 3: { // prev
              g.FillRectangle(SystemBrushes.ControlText, RectangleF.FromLTRB(image.Width * 0.1f, image.Height * 0.2f, image.Width * 0.2f, image.Height * 0.8f));
              var trianglePoints1 = new[] {
                new PointF(image.Width * 0.2f, image.Height * 0.5f),
                new PointF(image.Width * 0.5f, image.Height * 0.8f),
                new PointF(image.Width * 0.5f, image.Height * 0.2f),
              };
              g.FillPolygon(SystemBrushes.ControlText, trianglePoints1);
              var trianglePoints2 = new[] {
                new PointF(image.Width * 0.5f, image.Height * 0.5f),
                new PointF(image.Width * 0.8f, image.Height * 0.8f),
                new PointF(image.Width * 0.8f, image.Height * 0.2f),
              };
              g.FillPolygon(SystemBrushes.ControlText, trianglePoints2);
              break;
            }

            case 4: { // next
              using (var flipped = (Bitmap)imageButtonPrev.Clone()) {
                flipped.RotateFlip(RotateFlipType.RotateNoneFlipX);
                g.DrawImageUnscaled(flipped, 0, 0);
              }
              break;
            }
          }
        }
      }
    }

    private static Bitmap imageButtonPlay;
    private static Bitmap imageButtonStop;
    private static Bitmap imageButtonPause;
    private static Bitmap imageButtonPrev;
    private static Bitmap imageButtonNext;

    /*
     * events
     */
    public event EventHandler PlayDone;
    public event EventHandler TrackChanged;
    public event EventHandler PrevButtonClick;
    public event EventHandler NextButtonClick;

    /*
     * properties
     */
    public TimeSpan Position {
      get
      {
        RejectDisposed();
        if (player == null)
          return TimeSpan.MinValue;
        else
          return player.Position;
      }
      set
      {
        RejectDisposed();
        if (player != null)
          player.Position = value;
      }
    }

    public Track Track {
      get
      {
        RejectDisposed();
        return track;
      }
      set
      {
        RejectDisposed();
        OnTrackChanged(value);
      }
    }

    public string BgmSourcePath {
      get { return bgmSourcePath; }
      set
      {
        if (value == bgmSourcePath)
          return;

        bgmSourcePath = value;

        Stop();

        buttonPlay.Enabled = File.Exists(bgmSourcePath);

        waveFormView.BgmSourcePath = BgmSourcePath;
      }
    }

    public int Volume {
      get { return trackBarVolume.Value; }
      set { trackBarVolume.Value = value; }
    }

    public bool Repeat {
      get { return checkBoxRepeat.Checked; }
      set
      {
        RejectDisposed();

        checkBoxRepeat.Checked = value;

        if (player != null)
          player.Repeat = value;
      }
    }

    public bool IsPlaying {
      get { return isPlaying; }
    }

    public bool ShowPrevNextButtons {
      get { return showPrevNextButtons; }
      set
      {
        if (showPrevNextButtons != value)
          SetPrevNextButtonVisibility(value);
      }
    }

    /*
     * construction/destruction
     */
    public Player()
    {
      InitializeComponent();

      player = PcmStreamPlayer.Create();
      player.PlayDone += Player_PlayDone;

      waveFormView.SeekBarMoved += delegate(object sender, SeekBarMovedEventArgs e) {
        if (player == null || !player.Playing)
          e.Cancel = true;
        else
          player.Position = e.SeekBarPosition;
      };

      trackBarVolume.Minimum = MinVolume;
      trackBarVolume.Maximum = MaxVolume;
      trackBarVolume.SmallChange = (MaxVolume - MinVolume) / 100;
      trackBarVolume.LargeChange = (MaxVolume - MinVolume) / 10;
      trackBarVolume.Value = DefaultVolume;
      trackBarVolume.ValueChanged += delegate {
        SetPlayerVolume();
      };

      buttonPlay.Click += delegate {
        if (player == null || !player.Playing)
          Play();
        else if (player.Pausing)
          Resume();
        else
          Pause();
      };

      buttonPlay.Image = imageButtonPlay;
      buttonPlay.ImageAlign = ContentAlignment.MiddleCenter;

      buttonStop.Click += delegate { Stop(); };
      buttonStop.Image = imageButtonStop;
      buttonStop.ImageAlign = ContentAlignment.MiddleCenter;
      buttonStop.Enabled = false;

      buttonPrev.Click += delegate { OnPrevButtonClick(EventArgs.Empty); };
      buttonPrev.Image = imageButtonPrev;
      buttonPrev.ImageAlign = ContentAlignment.MiddleCenter;

      buttonNext.Click += delegate { OnNextButtonClick(EventArgs.Empty); };
      buttonNext.Image = imageButtonNext;
      buttonNext.ImageAlign = ContentAlignment.MiddleCenter;

      SetPrevNextButtonVisibility(showPrevNextButtons);

      checkBoxRepeat.CheckedChanged += delegate {
        if (player != null)
          player.Repeat = checkBoxRepeat.Checked;
      };

      SetPlayerVolume();

      timer = new System.Timers.Timer(100);
      timer.Enabled = false;
      timer.AutoReset = true;
      timer.SynchronizingObject = labelPosition;
      timer.Elapsed += delegate { SetSeekBarPosition(); };
    }

    private void Player_PlayDone(object sender, EventArgs e)
    {
      DisableStopButton();

      if (isPlaying) {
        isPlaying = false;

        OnPlayDone(EventArgs.Empty);
      }
    }

    protected override void OnLoad(EventArgs e)
    {
      timer.Start();

      base.OnLoad(e);
    }

    private void DisposeInternal(bool disposing)
    {
      if (disposing) {
        if (!disposed) {
          Stop();

          if (timer != null) {
            timer.Stop();
            timer.Dispose();
            timer = null;
          }

          if (player != null) {
            player.PlayDone -= Player_PlayDone;
            player.Close();
            player = null;
          }

          if (stream != null) {
            stream.Close();
            stream = null;
          }

          disposed = true;
        }
      }
    }

    private void SetPrevNextButtonVisibility(bool visible)
    {
      showPrevNextButtons = visible;

      if (showPrevNextButtons) {
        buttonPrev.Visible = true;
        buttonNext.Visible = true;
        panelPlayerControls.Left = buttonNext.Right + 8;
      }
      else {
        buttonPrev.Visible = false;
        buttonNext.Visible = false;
        panelPlayerControls.Left = buttonPrev.Left;
      }
    }

    /*
     * public methods
     */
    public void Play()
    {
      RejectDisposed();

      if (track == null)
        return;

      buttonPlay.Image = imageButtonPause;
      buttonStop.Enabled = true;

      try {
        stream = track.GetStream(BgmSourcePath);
      }
      catch (FileNotFoundException) {
        throw;
      }
      catch (ArgumentException) {
        buttonPlay.Image = imageButtonPlay;
        buttonStop.Enabled = false;
        return;
      }

      SetPlayerVolume();

      isPlaying = true;

      player.PlayAsync(stream, stream.Format.GetWaveFormatEx(), checkBoxRepeat.Checked);
    }

    public void Stop()
    {
      RejectDisposed();

      if (player == null)
        return;

      DisableStopButton();

      isPlaying = false;

      player.Stop();
    }

    public void Pause()
    {
      RejectDisposed();

      if (player == null)
        return;

      buttonPlay.Image = imageButtonPlay;

      player.Pause();
    }

    public void Resume()
    {
      RejectDisposed();

      if (player == null)
        return;

      buttonPlay.Image = imageButtonPause;

      player.Resume();
    }

    /*
     * private methods
     */
    private void SetPlayerVolume()
    {
      if (player == null)
        return;

      var scale = (trackBarVolume.Value - trackBarVolume.Minimum) / (double)(trackBarVolume.Maximum - trackBarVolume.Minimum);

      player.Volume = (int)(100 * scale);
    }

    private void DisableStopButton()
    {
      if (InvokeRequired) {
        Invoke((MethodInvoker)DisableStopButton);
        return;
      }

      buttonPlay.Image = imageButtonPlay;
      buttonStop.Enabled = false;
    }

    private void SetSeekBarPosition()
    {
      if (player == null)
        return;

      var position = player.Position;

      labelPosition.Text = string.Format("{0}:{1:D2}.{2:D3}", (int)position.TotalMinutes, position.Seconds, position.Milliseconds);

      waveFormView.SeekBarPosition = position;
    }

    private void OnTrackChanged(Track newTrack)
    {
      if (track == newTrack)
        return;

      Stop();

      if (stream != null) {
        stream.Close();
        stream = null;
      }

      track = newTrack;

      waveFormView.Track = track;

      effectEditor.Effect = (track == null) ? null : track.Effect;

      OnTrackChanged(EventArgs.Empty);
    }

    protected virtual void OnTrackChanged(EventArgs e)
    {
      var ev = this.TrackChanged;

      if (ev != null)
        ev(this, e);
    }

    protected virtual void OnPlayDone(EventArgs e)
    {
      var ev = this.PlayDone;

      if (ev != null)
        ev(this, e);
    }

    protected virtual void OnPrevButtonClick(EventArgs e)
    {
      var ev = this.PrevButtonClick;

      if (ev != null)
        ev(this, e);
    }

    protected virtual void OnNextButtonClick(EventArgs e)
    {
      var ev = this.NextButtonClick;

      if (ev != null)
        ev(this, e);
    }

    private void RejectDisposed()
    {
      if (disposed)
        throw new ObjectDisposedException(GetType().Name);
    }

    private bool disposed = false;
    private PcmStreamPlayer player;
    private bool isPlaying = false;
    private bool showPrevNextButtons = true;
    private Track track = null;
    private string bgmSourcePath = null;
    private EffectAppliedBgmStream stream = null;
    private System.Timers.Timer timer = null;
  }
}
